; 
; Control program for HRPT receiver with 35MHz IF
; modified for 16f84 PIC processor by GWR September 2002
; Channel display modifications by Bob barnes G4BGQ
; deleted wrong frequency of FY1C/FY1D ( was 1700.3 !! MHz instead of 1700.5 MHz )
; included Meteosat frequencies 1691 MHz and 1694.5 MHz
; included remote control via RS232 on PORTB,0
; included switching of Alblas decoder via PORTA,3/4 from HRPT to PDUS or CHRPT respectively
; included serial channel data via PORTB,7
; Hardwere changes needed:
;- remove R/C oscillator, install XT oscillator 4MHz
;- connect PORTA,4 to 5V via 1kOhm
;- install inverting transistor buffer for RS232 input on PORTB,0
; Connect U4 pin 13 to J3 pin 5 for channel data o/p

;************************************************************************
; processor pic16f84                                                    *
; no code protection                                                    *
; power up timer enabled                                                *
; WDT disabled                                                          *
; XT oscillator 4 MHz                                                   *
;************************************************************************
;***** Declaring and configuring a microcontroller *****

        PROCESSOR 16f84
        #include "p16f84.inc"
        #include "bank.inc"	; Assistant files
	
	__CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;************************************************************************

;
;Define registers
;Variables start after 0x0c

	Cblock 0x0C	; Beginning of RAM
; interrupt context save 
        w_
        status_
	fsr_
	pclath_
;
; variables used in synthesizer part
;
	synpoint               ;points to current synth data
	datalen                ;how many bits in current data sequence
	chanstr                ;previous channel selection
	chansel                ;current channel selection on switch or external interface
	bitctr                 ;counts off bits in byte
	wordctr                ;counts off synth data words
	charbuff               ;store for synth data characters
; RS232
        RS_TEMP1
        RS_TEMP2
        TXD
        RXD
        stringlen
; FIFO
        fifo_char
        fifo_len
        fifo_start
        fifo_base
        fifo_buffer:.16     ; we need 16 cells
;wait
	WCYCLE
	PRESCwait
	endc

;
;Define synth connections
;

#define	syndata		PORTA,1		;data
#define	synclk		PORTA,2		;clock
#define	synle		PORTA,0		;load enable
#define upbank          PORTA,3         ;upper bank when high: HRPT or CHRPT
#define upmem           PORTA,4         ;PDUS or CHRPT when high
#define serchan         PORTB,7         ;Serial data O/P to display

;
;Define valid input values for channel selections
;

chan1		equ	D'126'		;01111110
chan2		equ	D'125'		;01111101
chan3		equ	D'123'		;01111011
chan4		equ	D'119'		;01110111
chan5		equ	D'111'		;01101111
chan6		equ	D'95'		;01011111
chan7		equ	D'63'		;00111111



;
;************************************************************************

	ORG     0x00	; Reset vector
  	goto    start

	ORG     0x04	; Interrupt vector
        goto    ISR     ; interrupt routine

;	
;************************************************************************
;Data tables
;due to the paging mysteries of the PIC family I put the table on top of page
;		

getdata		addwf	PCL,1		;offset into table
table
reset1		retlw	0		;data to enter reset mode
		retlw	D'8'
		retlw	D'96'

reset2		retlw	0		;data to leave reset mode
		retlw	D'8'
		retlw	D'32'

rcount		retlw	0		;reference counter data
		retlw	D'12'
		retlw	D'128'

;
;Frequency settings for 35Mhz IF
;Frequency in comments is actual receive frequency.  The data programmes
;the synth to 35MHz below the receive frequency.
;

ch2		retlw	D'8'		;1698		HRPT
		retlw	D'30'
		retlw	D'200'

ch3		retlw	D'8'		;1700.4		CHRPT
		retlw	D'33'
		retlw	D'200'

ch4		retlw	D'8'		;1702.5		HRPT
		retlw	D'36'
		retlw	D'104'

ch5		retlw	D'8'		;1694.5		PDUS2
		retlw	D'26'
		retlw	D'104'

ch6		retlw	D'8'		;1691		PDUS1
		retlw	D'22'
		retlw	D'8'

ch7		retlw	D'8'		;1707		HRPT
		retlw	D'42'
		retlw	D'8'

;************************************************************************
	#include "wait.inc"

;
;select channel via RS232
RS_chantabl

;to avoid system hang when serial data stops prematurely we continuously poll for new switch position		
		movfw	PORTB		;get channel select data to W
		andlw	H'7E'		;mask off msb and lsb ( which will now be used for RS232 serial input )
		addlw	1		;so we can use old mask words without any trouble
		movwf	chansel		;store channel selection
		xorwf	chanstr,0	;check for channel change on switch
		btfss	STATUS,Z
		goto	Switch		;switch has changed
;

	        movfw   fifo_len	;now check for serial input via interrupt
        	btfsc   STATUS,Z
		goto	RS_chantabl
	        clrf    PORTA           ;default decoder to HRPT in lower bank
                WAIT    0x80            ;wait for Alblas Eprom to load
        	call    FIFOget
		andlw	0x07
		addwf	PCL,1		;offset into table
		goto	channel7
		goto	channel2
		goto	channel3
		goto	channel4
		goto	channel5
		goto	channel6
		goto	channel7
		goto	channel7
			


;************************************************************************

	#include "rs232.inc"
;************************************************************************
;       Interrupt service fuer RS232

ISR     movwf   w_		; save complete context
        swapf   STATUS, w
        movwf   status_
        bcf     STATUS, RP0
	movfw	PCLATH
	movwf	pclath_
        movfw   FSR
	movwf	fsr_

        RECEIVE

ISRend
	movfw	fsr_		; restore context
        movwf   FSR
	movfw	pclath_
	movwf	PCLATH
        swapf   status_,w
        movwf   STATUS
        swapf   w_,f
        swapf   w_,w
        bcf     INTCON,INTF
        RETFIE

;************************************************************************
	

;
;Program starts here
;
;
;Configure port A for output

start
                WAIT    0x80            ;wait for Alblas Eprom to load
		BANK1
	        movlw   0xe0            ; output on PortA 0,1,2,3,4
		movwf	TRISA
		BANK0
	        clrf    PORTA
		BANK1
	        movlw   0x7E           ; output on PortB 7
		movwf	TRISB
		BANK0
	        clrf    PORTB
	        RS232init       ; RS232


;
;Reset the synth and set to channel 6 (1707MHz)
;

;
;First put synth in reset mode
;

		movlw	D'20'		;set data length count
		movwf	datalen
		movlw	reset1-table	;point to reset1 data
		movwf	synpoint
		call	synthdata

		;
		;Then set the reference divider
		;

		movlw	D'21'
		movwf	datalen
		movlw	rcount-table	;point to R counter data
		movwf	synpoint
		call 	synthdata

		;
		;Then set the default channel 1707MHz
		;

		movlw	D'21'
		movwf	datalen
		movlw	ch6-table	;point to channel 6 data
		movwf	synpoint
		call	synthdata

		;
		;Return from reset mode
		;

		movlw	D'20'
		movwf	datalen
		movlw	reset2-table
		movwf	synpoint
		call 	synthdata

		movlw	0		;set initial channel store value
		movwf	chanstr

;
;Enter the main program loop
;Scan port A and look for change of input value.
;If any change, re-program the synth.
;If no change, look for legal serial input,re-program the synth accordingly.
;

loop
		movfw	PORTB		;get channel select data to W
		andlw	H'7E'		;mask off msb and lsb ( which will now be used for RS232 serial input )
		addlw	1		;so we can use old mask words without any trouble
		movwf	chansel		;store channel selection
		xorwf	chanstr,0	;check for channel change on switch
		btfss	STATUS,Z
		goto	Switch		;switch has changed
;
	        movfw   fifo_len	;now check for serial input via interrupt
        	btfsc   STATUS,Z
		goto	loop		;we did not receive any character

;rs232 service				;legal strings are "Chx" where x=0,1,2,3,4,5,6,7
	        call    FIFOget
        	movlw   'C'		;string must start with 'C'
        	subwf   fifo_char,w
        	bz      RS_h
        	goto    loop

RS_h

;to avoid system hang when serial data stops prematurely we continuously poll for new switch position		
		movfw	PORTB		;get channel select data to W
		andlw	H'7E'		;mask off msb and lsb ( which will now be used for RS232 serial input )
		addlw	1		;so we can use old mask words without any trouble
		movwf	chansel		;store channel selection
		xorwf	chanstr,0	;check for channel change on switch
		btfss	STATUS,Z
		goto	Switch		;switch has changed
;

	        movfw   fifo_len	;now check for serial input via interrupt
        	btfsc   STATUS,Z
		goto	RS_h
        	call    FIFOget
        	movlw   'h'		;next character must be 'h'
        	subwf   fifo_char,w
        	bz      RS_chantabl	;no further syntax check, we just jump
        	goto    loop


;************************************************************************
;
;select channel via front panel switch


Switch
	        clrf    PORTA           ;default decoder to HRPT in lower bank
                WAIT    0x80            ;wait for Alblas Eprom to load
                movf    chansel,0       ;store the changed channel
		movwf	chanstr


loop1		movf	chanstr,0	;get channel to W
		xorlw	chan2		;check for channel 2
		btfss	STATUS,Z
		goto	loop2		;go if not chan 2..			
		

channel2	bsf	PORTB, 7	;Channel 1 serial data
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		movlw	ch2-table	;else point to chan2 data	;HRPT 1698	
		goto	loop7

loop2		movf	chanstr,0	;get channel to W
		xorlw	chan3		;check for channel 3
		btfss	STATUS,Z
		goto	loop3		;go if not chan 3..
	        bsf     PORTA,3         ;switch alblas decoder to chrpt	;C-HRPT 1700.4
                bsf     PORTA,4         ;in upper bank & upper mem
		
	
channel3 	bsf	PORTB, 7	;Channel 2 serial data
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1		
		movlw	ch3-table	;else point to chan3 data						
		goto	loop7

loop3		movf	chanstr,0	;get channel to W
		xorlw	chan4		;check for channel 4	
		btfss	STATUS,Z
		goto	loop4		;go if not chan 4..
		
channel4        bsf	PORTB, 7	;Channel 3 serial data
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1		
		movlw   ch4-table       ;else point to chan4 data	;HRPT 1702.5			
		goto	loop7

loop4		movf	chanstr,0	;get channel to W
		xorlw	chan5		;check for channel 5
		btfss	STATUS,Z
		goto	loop5		;go if not chan 5..
		
channel5        bsf     PORTA,4         ;PDUS in lower bank & upper mem	;PDUS 1694.5
		bsf	PORTB, 7	;Channel 4 serial data
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7		
                movlw   ch5-table       ;else point to chan5 data		
		goto	loop7

loop5		movf	chanstr,0	;get channel to W
		xorlw	chan6		;check for channel 6
		btfss	STATUS,Z
		goto	loop6		;go if not chan 5..

channel6        bsf     PORTA,4         ;PDUS in lower bank & upper mem	;PDUS 1691		
		bsf	PORTB, 7	;Channel 5 serial data
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7		
                movlw   ch6-table       ;else point to chan6 data		
		goto	loop7

channel7
loop6		bsf	PORTB, 7	;Channel 6 serial data
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		WAIT	0x1
		bsf	PORTB, 7
		WAIT	0x1
		BCF	PORTB, 7
		movlw	ch7-table	;else point to chan7 data	;HRPT 1707
			
		
		

loop7		movwf	synpoint	;store table pointer
		movlw	D'21'		;set data length count
		movwf	datalen
                WAIT    0x40            ;wait for Alblas Eprom to load
		call	synthdata
		goto	loop

;
;Send channel or reset data to the synthesizer.
;Following code is entered with synpoint and datalen set.
;

synthdata	bcf	synclk		;clear clock bit
		bcf	synle		;clear load enable

synthdata1	movlw	D'8'		;set bit counter
		movwf	bitctr
		movf	synpoint,0	;put table offset pointer in W
		call	getdata		;get character from table
		movwf	charbuff	;temp store data byte
		incf	synpoint,1	;point to next data byte

synthdata2	btfsc	charbuff,7	;check if msb clear..
		goto	synthdata3	;if not go and set data bit..
		bcf	syndata		;else clear data bit..
		goto	synthdata4	;and then send data bit

synthdata3	bsf	syndata		;set data bit

synthdata4	nop
		bsf	synclk		;pulse clock high..
		bcf	synclk		;and then low
		rlf	charbuff,1	;move to next data bit
		decf	datalen,1	;dec the length count..
		decfsz	bitctr,1	;and the bit/byte count
					
		goto	synthdata5	;if bit/byte count not zero..
					;go test the length count..
		goto	synthdata1	;else get the next byte

synthdata5	movf	datalen,1	;test length count using Z
		btfss	STATUS,Z	;if not zero..
		goto	synthdata2	;process next bit..
		bsf	synle		;else send synth LE
		bcf	synle
		retlw	0
		


;************************************************************************
	End		; End of program






